#define RESOLUTION    10
#define ROTATETIME    (RESOLUTION*130/60)
#define PINGBACKWAIT  (40)

#define LED_PIN     PIN_PD3
#define ServoPin    PIN_PB2
#define Sonar_Ping  PIN_PB0
#define Sonar_Pong  PIN_PB1

#define M1_En PIN_PD5
#define M2_En PIN_PD6
#define M1_1  PIN_PC1
#define M1_2  PIN_PC0
#define M2_1  PIN_PC2
#define M2_2  PIN_PC3

//#define ANGLE_MULTP 4.93827
#define ANGLE_MULTP 4.3

#include <Servo.h>

const int WAITTIME = PINGBACKWAIT + ROTATETIME;

Servo myservo;

long measureDist() {
  long dur;
  digitalWrite(Sonar_Ping, LOW);
  delayMicroseconds(2);
  digitalWrite(Sonar_Ping, HIGH);
  delayMicroseconds(8);
  digitalWrite(Sonar_Ping, LOW);
  dur=pulseIn(Sonar_Pong, HIGH, 30000);
  if(dur==0)dur=30000;
  if(dur < 5){
    delay(100);
    dur = measureDist();
  }
  return (dur / 29 / 2);
}

void control_forward(uint8_t move_speed, float lbr_ratio = 1) {
  lbr_ratio *= 1;
  digitalWrite(M1_1, HIGH);
  digitalWrite(M1_2, LOW);
  digitalWrite(M2_1, HIGH);
  digitalWrite(M2_2, LOW);
  if (lbr_ratio > 1) {
    analogWrite(M2_En, move_speed);
    analogWrite(M1_En, int(move_speed / lbr_ratio));
    //    Serial.print("Right: ");
    //    Serial.print(move_speed);
    //    Serial.print(" Left: ");
    //    Serial.println(int(move_speed / lbr_ratio));
  } else {
    analogWrite(M2_En, int(move_speed * lbr_ratio));
    analogWrite(M1_En, move_speed);
    //    Serial.print("Right: ");
    //    Serial.print(int(move_speed * lbr_ratio));
    //    Serial.print(" Left: ");
    //    Serial.println(move_speed);
  }
}

void control_backward(uint8_t move_speed) {
  digitalWrite(M1_1, LOW);
  digitalWrite(M1_2, HIGH);
  digitalWrite(M2_1, LOW);
  digitalWrite(M2_2, HIGH);
  analogWrite(M1_En, move_speed);
  analogWrite(M2_En, move_speed);
}

void control_rawturn(int turntype, uint8_t move_speed = 255) {
  if (turntype > 0) {
    digitalWrite(M1_1, HIGH);
    digitalWrite(M1_2, LOW);
    digitalWrite(M2_1, LOW);
    digitalWrite(M2_2, LOW);
  } else if (turntype < 0) {
    digitalWrite(M1_1, LOW);
    digitalWrite(M1_2, LOW);
    digitalWrite(M2_1, HIGH);
    digitalWrite(M2_2, LOW);
  } else {
    digitalWrite(M1_1, HIGH);
    digitalWrite(M1_2, LOW);
    digitalWrite(M2_1, HIGH);
    digitalWrite(M2_2, LOW);
  }
  analogWrite(M1_En, move_speed);
  analogWrite(M2_En, move_speed);
  delay(int(ANGLE_MULTP));
  control_stop();
}

void control_turn(int16_t angle, uint8_t move_speed = 255) {
  if (angle > 0) {
    digitalWrite(M1_1, HIGH);
    digitalWrite(M1_2, LOW);
    digitalWrite(M2_1, LOW);
    digitalWrite(M2_2, HIGH);
  } else if (angle < 0) {
    digitalWrite(M1_1, LOW);
    digitalWrite(M1_2, HIGH);
    digitalWrite(M2_1, HIGH);
    digitalWrite(M2_2, LOW);
  } else {
    digitalWrite(M1_1, HIGH);
    digitalWrite(M1_2, LOW);
    digitalWrite(M2_1, HIGH);
    digitalWrite(M2_2, LOW);
  }
  analogWrite(M1_En, move_speed);
  analogWrite(M2_En, move_speed);
  delay(abs(int(ANGLE_MULTP * angle)));
  control_stop();
}

void control_stop() {
  analogWrite(M1_En, 0);
  analogWrite(M2_En, 0);
}

void setup() {
  Serial.begin(115200);
  myservo.attach(ServoPin);
  pinMode(LED_PIN, OUTPUT);
  pinMode(Sonar_Ping, OUTPUT);
  pinMode(Sonar_Pong, INPUT);
  pinMode(M1_En, OUTPUT);
  pinMode(M2_En, OUTPUT);
  pinMode(M1_1, OUTPUT);
  pinMode(M1_2, OUTPUT);
  pinMode(M2_1, OUTPUT);
  pinMode(M2_2, OUTPUT);
}

void keepRange(){
  long range = measureDist();
  while(range < 300){
    if(range < 60){
      control_backward(255);
      while(measureDist() >=65);
    }else if(range > 75){
      control_forward(255);
      while(measureDist() <= 70);
    }else{
      control_stop();
    }
    range = measureDist();
  }
  control_stop();
}

void lookingNearestObj(){
  long right = 500, left=500, center=500;

  bool led = 0;
  while(right > 290 && left > 290 && center > 290){
    digitalWrite(LED_PIN, led);
    led ^= 1;

    myservo.write(30);
    delay(60*110/60*1.3);
    right = measureDist();
    
    myservo.write(150);
    delay(120*110/60*1.3);
    left = measureDist();

    myservo.write(90);
    delay(60*110/60*1.3);
    center = measureDist();
  }
  digitalWrite(LED_PIN, 0);
  if(right < left){
    control_turn(60);
  }else if(left < right){
    control_turn(-60);
  }

}

void loop() {
  myservo.write(90);
  delay(90*130/60*1.4);
  keepRange();
  lookingNearestObj();
}
